/******** <lalVerbatim file="BlockBuildOutputCV"> ********
Author: Tibbits, M. M.
$Id: BlockBuildOutput.c,v 1.4 2003/01/23 00:47:25 tibbits Exp $
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="BlockBuildOutputC">
\subsection{Module \texttt{BlockBuildOutput.c}}
\label{s:BlockBuildOutput.c}

Routine that builds the output from the result vector stored in the
parameter structure for insertion into the metadata database and frames.

\subsubsection*{Prototypes}
\input{BlockBuildOutputCP}
\index{\texttt{BlockBuildOutput()}}

\subsection*{Description}

Routine that builds the output from the result vector stored in the
parameter structure for insertion into the metadata database and frames.


\subsection*{Tables Used}

\begin{itemize}
\item \texttt{search::summary}
\item \texttt{summ::value}
\end{itemize}

\subsection*{Uses}
 
\begin{itemize}
\item \texttt{LALMalloc}
\item \texttt{LALCalloc}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockBuildOutputCV}}

*************************************************** </lalLaTeX> */

#include "Block.h"

NRCSID( BLOCKBUILDOUTPUTC, "$Id: BlockBuildOutput.c,v 1.4 2003/01/23 00:47:25 tibbits Exp $");

/************************ <lalVerbatim file="BlockBuildOutputCP"> */
void BuildOutput
(
	LALStatus		*status,
	LALSearchOutput		*output,
	BlockSearchParams	*params
)
/*********************************************** </lalVerbatim> */
{
	dataBase	*dbEntry	= NULL;
	outPut		*outputPtr	= NULL;
const	CHAR		*PROGRAM	= "Block Normal Search";

	INITSTATUS( status, "BlockBuildOutput", BLOCKBUILDOUTPUTC );

	output->numOutput = 2;

	/*  Allocate Memory  */
	output->result = outputPtr = LALMalloc( (output->numOutput) * sizeof(outPut) );
	if ( !output->result )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	outputPtr[0].indexNumber	= 0;
	outputPtr[0].search		= experimental;
	outputPtr[0].significant	= 1;	/* 1 - IS significant */
	outputPtr[0].states		= 0;
	outputPtr[0].optional		= 0;

	/*
	 * Start a linked list of database entries - the head of the list
	 * starts with the "results" field
	 */

	outputPtr[0].results = dbEntry = LALCalloc(1, sizeof(dataBase));
	if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	dbEntry->previous = NULL;



	/* Shared Object Name */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "shared_object", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = LALMalloc(sizeof(CHAR)*(strlen(PROGRAM) + 1) );
	strcpy(dbEntry->rows.chars, PROGRAM);

	dbEntry->rowDimensions = LALMalloc(sizeof(UINT4));

	/* Don't include terminating null */
	dbEntry->rowDimensions[0] = strlen(PROGRAM);

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}


	/* LAL CVS tag */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "lal_cvs_tag", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(LAL_CVS_TAG)+1) * sizeof(CHAR) );
	if ( !dbEntry->rows.chars )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	strcpy( dbEntry->rows.chars, LAL_CVS_TAG);

	dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
	if ( !dbEntry->rowDimensions )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	*(dbEntry->rowDimensions) = strlen(LAL_CVS_TAG);

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}


	/* LALWrapper CVS tag */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "lalwrapper_cvs_tag", dbNameLimit );

	dbEntry->type		= char_s_ptr;
	dbEntry->numberRows	= 1;

	dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(LAL_WRAPPER_CVS_TAG)+1) * sizeof(CHAR) );
	if ( !dbEntry->rows.chars )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	strcpy( dbEntry->rows.chars, LAL_WRAPPER_CVS_TAG);

	dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
	if ( !dbEntry->rowDimensions )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	*(dbEntry->rowDimensions) = strlen(LAL_WRAPPER_CVS_TAG);

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;

        if ( !dbEntry )
	{
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}



	/* In Start time */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
	strncpy( dbEntry->columnName, "in_start_time", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	*(dbEntry->rows.int4s) = params->times.startSec;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {                
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }


	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_start_time_ns", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );

	*(dbEntry->rows.int4s) = params->times.startNan;
	dbEntry->rowDimensions = NULL;
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
	if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}



	/*  In End time  */
	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_end_time", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	*(dbEntry->rows.int4s) = params->times.stopSec;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

	strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
	strncpy( dbEntry->columnName, "in_end_time_ns", dbNameLimit );

	dbEntry->type       = int_4s;
	dbEntry->numberRows = 1;
	dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }


	*(dbEntry->rows.int4s) = params->times.stopNan;
	dbEntry->rowDimensions = NULL;

	dbEntry->next = LALCalloc(1, sizeof(dataBase));
	dbEntry->next->previous = dbEntry;
	dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }



        /* Out Start time */
        strncpy( dbEntry->tableName, "search_summary", dbNameLimit);
        strncpy( dbEntry->columnName, "out_start_time", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.startSec;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_start_time_ns", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.startNan;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }



        /* Out End time */
        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_end_time", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.stopSec;
        dbEntry->rowDimensions = NULL;

        dbEntry->next = LALCalloc(1, sizeof(dataBase));
        dbEntry->next->previous = dbEntry;
        dbEntry = dbEntry->next;
        if ( !dbEntry )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        strncpy( dbEntry->tableName, "search_summary", dbNameLimit );
        strncpy( dbEntry->columnName, "out_end_time_ns", dbNameLimit );

        dbEntry->type       = int_4s;
        dbEntry->numberRows = 1;
        dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
        if ( !dbEntry->rows.int4s )
        {
                ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
        }

        *(dbEntry->rows.int4s) = params->times.stopNan;
        dbEntry->rowDimensions = NULL;

        /* Terminate the list */
        dbEntry->next = NULL;

	if ( params->resultsToStore )
	{
		outputPtr[1].indexNumber	= 0;
		outputPtr[1].search		= experimental;
		outputPtr[1].significant	= 1;
		outputPtr[1].states		= 0;
		outputPtr[1].optional		= 0;

		outputPtr[1].results = params->resultsToStore;

		params->resultsToStore = NULL;
	}

	RETURN(status);
}


/************************ <lalVerbatim file="BlockBuildOutputCP"> */
void AddResults
(
	LALStatus		*status,
	LALSearchOutput		*output,
	BlockSearchParams	*params
)
/*********************************************** </lalVerbatim> */
{
	INT4		iterator = 0;
	dataBase	*dbEntry = NULL;
	BOOLEAN		firstTime = 1;
	BlockResult	*current = NULL;

	INITSTATUS( status, "AddResults", BLOCKBUILDOUTPUTC );

	if ( params->resultsToStore )
	{
		params->lastResultStored->next = dbEntry = LALCalloc(1, sizeof(dataBase));
		dbEntry->previous = params->lastResultStored;
	}
	else
	{
		params->resultsToStore = dbEntry = LALCalloc(1, sizeof(dataBase));
		params->resultsToStore->previous = NULL;
	}

	if ( !dbEntry )
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

    for(iterator = 0; iterator < params->data->levels; iterator++)	
	while( params->results[iterator] != NULL )
	{
		current = params->results[iterator];
		params->results[iterator] = params->results[iterator]->next;

                if( !firstTime )
		{
			dbEntry->next = LALCalloc(1, sizeof(dataBase));
			dbEntry->next->previous = dbEntry;
			dbEntry = dbEntry->next;
		
			if ( !dbEntry )
			{
				ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
			}
		}

		firstTime = 0;

		/*  Search Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit );
		strncpy( dbEntry->columnName, "search", dbNameLimit );

		dbEntry->type           = char_s_ptr;
		dbEntry->numberRows     = 1;

		dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen("Block Search")+1) * sizeof(CHAR) );
		if ( !dbEntry->rows.chars )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strcpy( dbEntry->rows.chars, "Block Search");
		dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
		if ( !dbEntry->rowDimensions )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rowDimensions) = strlen("Block Search");

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  IFO Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit );
		strncpy( dbEntry->columnName, "ifo", dbNameLimit );

		dbEntry->type           = char_s_ptr;
		dbEntry->numberRows     = 1;

		dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(params->ifo)+1) * sizeof(CHAR) );
		if ( !dbEntry->rows.chars )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strcpy( dbEntry->rows.chars, params->ifo );
		dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
		if ( !dbEntry->rowDimensions )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rowDimensions) = strlen(params->ifo);

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



                /*  Channel Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit );
		strncpy( dbEntry->columnName, "channel", dbNameLimit );

		dbEntry->type		= char_s_ptr;
		dbEntry->numberRows	= 1;

		dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(params->channel)+1) * sizeof(CHAR) );
		if ( !dbEntry->rows.chars )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strcpy( dbEntry->rows.chars, params->channel );
		dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
		if ( !dbEntry->rowDimensions )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rowDimensions) = strlen(params->channel);

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/* Start time */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit);
		strncpy( dbEntry->columnName, "start_time", dbNameLimit );

		dbEntry->type       = int_4s;
		dbEntry->numberRows = 1;
		dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
		if ( !dbEntry->rows.int4s )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}


/****************************************************************************
 *
 *   Incorrect - should calculate times truly.  this is faked.
 *
 *****************************************************************************/

		*(dbEntry->rows.int4s) = current->startTime;
		dbEntry->rowDimensions = NULL;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit );
		strncpy( dbEntry->columnName, "start_time_ns", dbNameLimit );

		dbEntry->type       = int_4s;
		dbEntry->numberRows = 1;
		dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
		if ( !dbEntry->rows.int4s )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

/****************************************************************************
 *
 *   Incorrect - should calculate times truly.  this is faked.
 *
 ****************************************************************************/

		*(dbEntry->rows.int4s) = current->startTime;
		dbEntry->rowDimensions = NULL;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Duration Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit );
		strncpy( dbEntry->columnName, "duration", dbNameLimit );

		dbEntry->type	    = int_4s;
		dbEntry->numberRows = 1;
		dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
		if ( !dbEntry->rows.int4s )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}
		
		*(dbEntry->rows.int4s) = current->startTime;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Variance Entry  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit);
		strncpy( dbEntry->columnName, "amplitude", dbNameLimit );

		dbEntry->type       = real_4;
		dbEntry->numberRows = 1;
		dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) );
		if ( !dbEntry->rows.real4 )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rows.real4) = (REAL4)(log(current->variance));

		if ( *(dbEntry->rows.real4) > 10000 )
			*(dbEntry->rows.real4) = -1;

		dbEntry->rowDimensions = NULL;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Signal to Noise Entry  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit);
		strncpy( dbEntry->columnName, "snr", dbNameLimit );

		dbEntry->type       = real_4;
		dbEntry->numberRows = 1;
		dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) );
		if ( !dbEntry->rows.real4 )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rows.real4) = (REAL4)(log(current->value));

		if ( *(dbEntry->rows.real4) > 10000 )
			*(dbEntry->rows.real4) = -1;

		dbEntry->rowDimensions = NULL;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Confidence  */
		strncpy( dbEntry->tableName, "sngl_unmodeled", dbNameLimit);
		strncpy( dbEntry->columnName, "confidence", dbNameLimit );

		dbEntry->type       = real_4;
		dbEntry->numberRows = 1;
		dbEntry->rows.real4 = LALMalloc( sizeof(REAL4) );
		if ( !dbEntry->rows.real4 )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rows.real4) = (REAL4)(log(current->rho2));
		
		if ( *(dbEntry->rows.real4) > 10000 )
			*(dbEntry->rows.real4) = -1;

		dbEntry->rowDimensions = NULL;

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Event Id Tag??  */


		/*  IFO Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled_v", dbNameLimit );
		strncpy( dbEntry->columnName, "ifo", dbNameLimit );

		dbEntry->type		= char_s_ptr;
		dbEntry->numberRows	= 1;

		dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen(params->ifo)+1) * sizeof(CHAR) );
		if ( !dbEntry->rows.chars )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strcpy( dbEntry->rows.chars, params->ifo );
		dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
		if ( !dbEntry->rowDimensions )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rowDimensions) = strlen(params->ifo);

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Name Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled_v", dbNameLimit );
		strncpy( dbEntry->columnName, "name", dbNameLimit );

		dbEntry->type           = char_s_ptr;
		dbEntry->numberRows     = 1;

		dbEntry->rows.chars = (CHAR *) LALMalloc( (strlen("Wavelet->level")+1) * sizeof(CHAR) );
		if ( !dbEntry->rows.chars )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		strcpy( dbEntry->rows.chars, "Wavelet->level" );
		dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
		if ( !dbEntry->rowDimensions )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rowDimensions) = strlen("Wavelet->level");

		dbEntry->next = LALCalloc(1, sizeof(dataBase));
		dbEntry->next->previous = dbEntry;
		dbEntry = dbEntry->next;
		if ( !dbEntry )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}



		/*  Value Tag  */
		strncpy( dbEntry->tableName, "sngl_unmodeled_v", dbNameLimit );
		strncpy( dbEntry->columnName, "value", dbNameLimit );

		dbEntry->type       = int_4s;
		dbEntry->numberRows = 1;
		dbEntry->rows.int4s = LALMalloc( sizeof(INT4) );
		if ( !dbEntry->rows.int4s )
		{
			ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
		}

		*(dbEntry->rows.int4s) = iterator;

		dbEntry->rowDimensions = NULL;

		LALFree( current );
	}

	dbEntry->next = NULL;

	params->lastResultStored = dbEntry;

	RETURN(status);
}
