/******** <lalVerbatim file="BlockFinalizeSearchCV"> ********
Author: Tibbits, M. M.
$Id: BlockFinalizeSearch.c,v 1.3 2003/01/22 23:03:09 tibbits Exp $
********* </lalVerbatim> ********/

/*************************** <lalLaTeX file="BlockFinalizeSearchC">
\subsection{Module \texttt{BlockFinalizeSearch.c}}
\label{s:BlockFinalizeSearch.c}

Routine that cleans up after the stochastic search.

\subsubsection*{Prototypes}
\input{BlockFinalizeSearchCP}
\index{\texttt{LALFinalizeSearch()}}

\subsection*{Description}

Cleans up after the block normal search--i.e., frees memory previously 
allocated in the \texttt{LALConditionData} routine. 

\subsection*{Algorithms}
  
\subsection*{Uses}

\begin{itemize}
\item \texttt{LALDDestroyVectorSequence} 
\item \texttt{LALDDestroyVector} 
\item \texttt{LALFree}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockFinalizeSearchCV}}

*************************************************** </lalLaTeX> */

#include "Block.h"

NRCSID( BLOCKFINALIZESEARCHC, "$Id: BlockFinalizeSearch.c,v 1.3 2003/01/22 23:03:09 tibbits Exp $");

/************************ <lalVerbatim file="BlockFinalizeSearchCP"> */
void LALFinalizeSearch
(
	LALStatus	*status,
	void		**searchParams
)
/*********************************************** </lalVerbatim> */
{
	BlockSearchParams	*params;
	BlockResult		*tempResult;
	waveletSegment		*tempPointer;
	INT4			iterator;

	INITSTATUS( status, "LALFinalizeSearch", BLOCKFINALIZESEARCHC );
	ATTATCHSTATUSPTR( status );

	params = ((BlockSearchParams*)(*searchParams));

	tempPointer	= NULL;
	tempResult	= NULL;

	if (params->data != NULL)
	{
		for(iterator = 0; iterator < params->data->levels; iterator++ )
		{
			if( params->results != NULL )
			{
				while ( params->results[iterator] != NULL )
				{
					tempResult = params->results[iterator];
					params->results[iterator] = params->results[iterator]->next;
					LALFree ( tempResult );
				}

				LALFree( params->results );
			}
				
		}

		for( iterator = 0; iterator < params->data->levels; iterator++)
		{
			while( params->data->decomposition[iterator] != NULL)
			{
				tempPointer = params->data->decomposition[iterator];
				params->data->decomposition[iterator] = params->data->decomposition[iterator]->next;

				LALDDestroyVector( status->statusPtr, &tempPointer->data );
				CHECKSTATUSPTR( status );
				LALFree ( tempPointer );
			}
		}

		if ( params->data->decomposition != NULL )
			LALFree( params->data->decomposition );

		LALFree(params->data);
	}

	if ( params->prevResults != NULL )
		LALFree( params->prevResults );

	if ( params != NULL )
		LALFree(params);

	DETATCHSTATUSPTR( status );
	RETURN(status);
}
