/****************** <lalVerbatim file="BlockInitSearchCV">
Author: Tibbits, M. M.
$Id: BlockInitSearch.c,v 1.4 2003/01/23 00:47:25 tibbits Exp $
********************************* </lalVerbatim> */

/******************************* <lalLaTeX file="BlockInitSearchC">
\subsection{Module \texttt{BlockInitSearch.c}}
\label{s:BlockInitSearch.c}

Routine that checks the validity of the input parameters and initializes 
them.

\subsubsection*{Prototypes}
\input{BlockInitSearchCP}
\index{\texttt{LALInitSearch()}}

\subsection*{Description}

Routine that checks the validity of the input parameters and initializes
them.

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockInitSearchCV}}

*************************************************** </lalLaTeX> */

#include "Block.h"

NRCSID( BLOCKINITSEARCHC, "$Id: BlockInitSearch.c,v 1.4 2003/01/23 00:47:25 tibbits Exp $");

/************************ <lalVerbatim file="BlockInitSearchCP"> */
void LALInitSearch
(
	LALStatus		*status,
	void			**searchParams,
	LALInitSearchParams	*initSearchParams
)
/*********************************************** </lalVerbatim> */
{
	BlockSearchParams	*local;

	INITSTATUS( status, "LALInitSearch", BLOCKINITSEARCHC );

	local = LALMalloc(sizeof(BlockSearchParams));
	if(!local)
	{
		ABORT( status, BLOCKH_EALOC, BLOCKH_MSGEALOC );
	}

	/*  Turn on Debugging 
	lalDebugLevel = LALALLDBG;
*/
	/*  Initialize parameters  */
	local->searchMaster	= initSearchParams->nodeClass;
	local->rank		= 0;

	local->numberOfSequences	= 0;
	local->segmentsRemaining	= 0;
	local->resultsObtained		= 0;
	local->numSlaves		= 0;
	local->numOutput		= 0;

	local->lastResultStored	= NULL;
	local->resultsToStore	= NULL;
	
	local->prevResults	= NULL;
	local->results		= NULL;
	local->head		= NULL;
	local->tail		= NULL;
	local->data		= NULL;

	strcpy(local->channel, "");
	strcpy(local->ifo, "");

	ASSERT( searchParams, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	ASSERT( initSearchParams, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	ASSERT( initSearchParams->argv, status, BLOCKH_ENULL, BLOCKH_MSGENULL);

	ASSERT( initSearchParams->argc == 6, status, BLOCKH_EARG, BLOCKH_MSGEARG );

	ASSERT(!(strcmp( initSearchParams->argv[0], "-filterparams" )), status, BLOCKH_EARG, BLOCKH_MSGEARG);

	local->threshold	= atof( initSearchParams->argv[1] );
	local->maxVectorLength	= atoi( initSearchParams->argv[2] );
	local->minVectorLength	= atoi( initSearchParams->argv[3] );
	local->overlap		= atof( initSearchParams->argv[4] );
	local->overlapSwitch	= atoi( initSearchParams->argv[5] );

	*searchParams = local;

	RETURN(status);
}
