/******** <lalVerbatim file="BlockTreeCV"> ********
Author: Tibbits, M. M.
$Id: BlockTree.c,v 1.2 2003/01/23 00:47:25 tibbits Exp $
********* </lalVerbatim> ********/

/******************************* <lalLaTeX file="BlockTreeC">
\subsection{Module \texttt{BlockTree.c}}
\label{s:BlockTree.c}

Yeah tree algorithm.  WOO

\subsubsection*{Prototypes}
\input{BlockTreeCP}
\index{\texttt{BlockAddQ()}}
\index{\texttt{BlockDelQ()}}
\index{\texttt{BlockSplitLevel()}}

note: BlockAddQ & BlockDelQ are not LALSpec functions.

\subsection*{Description}

\subsection*{Algorithms}

\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockTreeCV}}

 *************************************************** </lalLaTeX> */

#include "Block.h"
#include "lal/LALMoment.h"

NRCSID( BLOCKTREEC, "$Id: BlockTree.c,v 1.2 2003/01/23 00:47:25 tibbits Exp $");

/*
 * Function BlockAddQ
 *
 * Adds the pointer newSegment to the tail of the Queue.
 * Not LALSpec Function  ( No status )
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
void BlockAddQ
(
	LALStatus	*status,
	waveletSegment	*newSegment,
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;

	INITSTATUS( status, "BlockTree", BLOCKTREEC );

	/*  Inititalize Variables  */
	params = ((BlockSearchParams*)(searchParams));

	newSegment->next = NULL;

	if( params->tail == NULL )
	{
		params->head = newSegment;
	}
	else
	{
		params->tail->next = newSegment;
	}

	params->tail = newSegment;

	RETURN(status);
}


/*
 * Function BlockDelQ
 *
 * Returns pointer to head segment and removes head segment from the
 * Queue.  If head is NULL, function returns NULL.
 * Not LALSpec Function  ( No status )
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
waveletSegment*	BlockDelQ
(
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;
	waveletSegment		*tmp;

	/*  Inititalize Variables  */
	params = ((BlockSearchParams*)(searchParams));

	if( params->head == NULL )
	{
		return( NULL );
	}

	tmp = params->head;

	if( params->head == params->tail )
	{
		params->head = params->tail = NULL;
	}
	else
	{
		params->head = params->head->next;
	}

	return( tmp );
}


/*
 * Function BlockSplitLevel
 *
 * Parse to split input segment into two separate pieces, left & right halves.
 * A segment will not be created if it would be smaller than
 * params->minVectorLength.  Input will not be split if rho2 isn't higher than
 * the threshold.
 *
 */

/************************ <lalVerbatim file="BlockTreeCP"> */
void BlockSplitLevel
(
	LALStatus	*status,
	waveletSegment	*input,
	void		*searchParams
)
/*********************************************** </lalVerbatim> */
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;
	waveletSegment		*tmp;
	BlockResult			*current;
	UINT4				iterator;

	INITSTATUS( status, "BlockTree", BLOCKTREEC );
	ATTATCHSTATUSPTR (status);

	/*  Inititalize Variables  */
	params	= ((BlockSearchParams*)(searchParams));
	input->left = input->right = NULL;
	tmp = NULL;
	
	/*  if we aren't above threshold, we're done.  */
	if( log(input->rho2) < params->threshold )
	{
		DETATCHSTATUSPTR( status );
		RETURN(status);
	}

	/*  Since we're above threshold, add current segment to slave result chain  */
	current = (BlockResult*)(LALMalloc ( sizeof(BlockResult)));
	if( !current)
	{
		ABORT (status, BLOCKH_EALOC, BLOCKH_MSGEALOC);
	}

	current->waveletLevel = input->level;

	/* We need to do something about segmentID */
	/*  current->segmentID;  */

	current->startTime = input->startTime;
	current->duration  = input->duration;
	current->value     = input->value;
	current->index     = input->index;
	current->rho2      = input->rho2;

	current->next = NULL;

	if( params->results[input->level] == NULL )
	{
		params->results[input->level] = current;
	}
	else
	{
		params->prevResults[input->level]->next = current;
	}

	params->prevResults[input->level] = current;

	params->numberResults++;

	current = NULL;

	/*  Chop Vector based on Input Parameters  */

	if( input->index > params->minVectorLength )
	{
		tmp = (waveletSegment*)(LALMalloc( sizeof( waveletSegment)));
		if( !tmp )
		{
			ABORT(status, BLOCKH_EDATA, BLOCKH_MSGEDATA);
		}

		input->left = tmp;
		
		/*  Initialize new Segment  */
		tmp->rho2  = tmp->index = tmp->value = 0;
		tmp->left  = NULL;
		tmp->right = NULL;
		tmp->next  = NULL;
		tmp->data  = NULL;

		/*  Copy Id information  */
		tmp->level = input->level;
		tmp->startTime = input->startTime;

		/*  Calculate Duration  */
		tmp->duration = input->index;

		if(( input->level != 0 )&&( input->level != (params->data->levels - 1)) )
		{
			tmp->duration *= pow(2,input->level);
		}
		else
		{
			if( input->level != 0 )
			{
				tmp->duration *= pow(2,input->level - 1);
			}
		}

		/*  Copy Data  */
		LALDCreateVector( status->statusPtr, &(tmp->data), input->index);
		CHECKSTATUSPTR( status );

		for( iterator = 0; iterator < input->index; iterator++ )
		{
			tmp->data->data[iterator] = input->data->data[iterator];
		}

		LALDMoment(status->statusPtr, &(tmp->variance), tmp->data, 2 );
		LALDMoment(status->statusPtr, &(tmp->mean), tmp->data, 1);

	}

	if( (input->data->length - input->index) > params->minVectorLength )
	{
		tmp = (waveletSegment*)(LALMalloc( sizeof( waveletSegment)));
		if( !tmp )
		{
			ABORT(status, BLOCKH_EDATA, BLOCKH_MSGEDATA);
		}

		input->right = tmp;

		/*  Initialize new Segment  */
		tmp->rho2 = tmp->index = tmp->value = 0;
                tmp->left  = NULL;
		tmp->right = NULL;
		tmp->next  = NULL;
		tmp->data  = NULL;

		/*  Copy Id information  */
		tmp->level = input->level;

		/*  Calculate startTime  */
		tmp->startTime = input->index;

		if(( input->level != 0 )&&( input->level != (params->data->levels - 1)) )
		{
			tmp->startTime *= pow(2,input->level);
		}
		else
		{
			if( input->level != 0 )
			{
				tmp->startTime *= pow(2,input->level - 1);
			}
		}

		tmp->startTime += input->startTime;

		/*  Calculate Duration  */
		tmp->duration = input->data->length - input->index;

		if(( input->level != 0 )&&( input->level != (params->data->levels - 1)) )
		{
			tmp->duration *= pow(2,input->level);
		}
		else
		{
			if( input->level != 0 )
			{
				tmp->duration *= pow(2,input->level - 1);
			}
		}

		/*  Copy Data  */
		LALDCreateVector( status->statusPtr, &(tmp->data), input->data->length - input->index);
		CHECKSTATUSPTR( status );

		for( iterator = input->index; iterator < input->data->length; iterator++ )
		{
			tmp->data->data[iterator - input->index] = input->data->data[iterator];
		}

		LALDMoment(status->statusPtr,&(tmp->variance), tmp->data, 2 );
		LALDMoment(status->statusPtr, &(tmp->mean), tmp->data, 1);
	}

	DETATCHSTATUSPTR(status);
	RETURN(status);
}
