/********************************** <lalVerbatim file="COHApplySearchCV">
Author: Julien Sylvestre
$Id: ApplySearch.c,v 1.5 2002/12/02 17:42:31 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/TFClusters.h>
#include <lal/Comm.h>
#include <InitSearch.h>
#include <ApplySearch.h>
#include <unistd.h>
#include <lal/TFCThresholds.h>
#include <strings.h>
#include <BuildDB.h>

NRCSID( APPLYSEARCHC, "power $Id: ApplySearch.c,v 1.5 2002/12/02 17:42:31 julien Exp $");

void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  static UINT4 alphaInd=0, deltaInd=0, psiInd=0, plusDotcrossInd=0, plus2crossInd=0, NDone = 0, NProcessed=0;
  static UINT4 N2Go = -1U;
  static UINT4 NBatch = -1U;
  static BOOLEAN NotDone = 1;
  static REAL8 *zePower = NULL;
  
  UINT4 i,j;
  INT4 myRank;
  CohereSearchParams *params = (CohereSearchParams*)applyParams->searchParams;
  InitExchParams           initExchParams;

  INITSTATUS (status, "LALApplySearch", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );

  /* check that the master is running on node rank zero */
  if ( params->searchMaster && myRank != 0 )
  {
    ABORT( status, APPLYSEARCHH_ERANK, APPLYSEARCHH_MSGERANK );
  }

  /* and that the slave is not running on node rank zero */
  if ( !params->searchMaster && myRank == 0 )
  {
    ABORT( status, APPLYSEARCHH_ERANK, APPLYSEARCHH_MSGERANK );
  }
 
  initExchParams.mpiComm   = *(applyParams->mpiParams->comm);
  initExchParams.myProcNum = myRank;


  if(params->searchMaster && N2Go == -1U)  {

    /* only do this the first time the master is called */
    MPI_Comm_size( *(applyParams->mpiParams->comm), &(params->numSlaves) );

    N2Go = params->numSlaves - 1;
    
    zePower = (REAL8 *)LALMalloc( params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross * sizeof(REAL8));
    if(!zePower) {
      ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }

    /* set NBatch so code reports every ~5% */
    NBatch = (UINT4)floor(0.05*(REAL4)(params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) / (REAL4)N2Go);

    /* make sure it scales well for large systems */
    if(NBatch * N2Go > params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) {
      NBatch = (UINT4)floor((REAL4)(params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) / (REAL4)N2Go);
    }

    if(NBatch == 0) {NBatch = 1;}

  }


  
  /* 
   *
   * the main engine starts here.  On the master 
   *
   */

  if(params->searchMaster) {

    UINT4 tmpU4, alpha, delta, psi, plusDotcross, plus2cross;

    if ( N2Go )
    {
      ExchParams *thisExch = NULL;

      /* wait for a message */
      LALInitializeExchange (status->statusPtr, &thisExch, NULL, &initExchParams);
      CHECKSTATUSPTR (status);
      
      /* case the exchange type */
      switch (thisExch->exchObjectType)
      {
	/* send a job to slave */
      case ExchDataList:

	if(plusDotcrossInd < params->NplusDotcross) {
	  UINT4 jcnt, n2do;

	  if(NProcessed + NBatch <= params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) {
	    n2do = NBatch;
	  } else {
	    n2do = params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross - NProcessed;
	  }

	  LALExchangeUINT4(status->statusPtr, &n2do, thisExch);
	  CHECKSTATUSPTR (status);

	  for(jcnt=0; jcnt<n2do; jcnt++) {
	    /* tell the slave alpha, beta, psi, ... */
	    LALExchangeUINT4(status->statusPtr, &alphaInd, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4(status->statusPtr, &deltaInd, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4(status->statusPtr, &psiInd, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4(status->statusPtr, &plus2crossInd, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4(status->statusPtr, &plusDotcrossInd, thisExch);
	    CHECKSTATUSPTR (status);

	    alphaInd++;
	    if(alphaInd == params->NAlpha) {
	      alphaInd = 0;
	
	      deltaInd++;
	      if(deltaInd == params->NDelta) {
		deltaInd = 0;
	
		psiInd++;
		if(psiInd == params->NPsi) {
		  psiInd = 0;

		  plus2crossInd++;
		  if(plus2crossInd == params->Nplus2cross) {
		    plus2crossInd = 0;

		    plusDotcrossInd++;
		  }
		}

	      }
	    }

	    NProcessed++;

	  }
	} else {

	  /* no more to do */
	  tmpU4 = -1U;
	  LALExchangeUINT4 (status->statusPtr, &tmpU4, thisExch);
	  CHECKSTATUSPTR (status);

	  N2Go--;
	}
	break;


	/* recieve results */
      case ExchEPEvent:

	{
	  UINT4 nResults, jc;

	  LALExchangeUINT4 (status->statusPtr, &nResults, thisExch);
	  CHECKSTATUSPTR (status);

	  for(jc=0;jc<nResults;jc++) {

	    LALExchangeUINT4 (status->statusPtr, &alpha, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4 (status->statusPtr, &delta, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4 (status->statusPtr, &psi, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4 (status->statusPtr, &plus2cross, thisExch);
	    CHECKSTATUSPTR (status);
	    LALExchangeUINT4 (status->statusPtr, &plusDotcross, thisExch);
	    CHECKSTATUSPTR (status);

	    LALExchangeREAL8 (status->statusPtr, 
			      zePower + alpha + delta * params->NAlpha + psi * params->NAlpha * params->NDelta + plus2cross * params->NAlpha * params->NDelta * params->NPsi + plusDotcross * params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross, 
			      thisExch);
	    CHECKSTATUSPTR (status);

	    NDone++;

	  }
	}
		
	break;

      case ExchFinished:
	break;

      case ExchBeacon:
	break;

      default: 
	ABORT( status, APPLYSEARCHH_EUEXT, APPLYSEARCHH_MSGEUEXT );
      }

      /* finish the exchange */
      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* 
       *
       * set fraction remaining
       *
       */

      output->numOutput = 0;
      output->result = NULL;
      output->fracRemaining = 1.0 - (REAL4)NDone / (REAL4)(params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross);
      output->notFinished = 1;

    }
   
    else {

      /* all the slaves are done */

      BuildOutput(status->statusPtr,
		  output,
		  zePower,
		  params);
      CHECKSTATUSPTR (status);
      
      LALFree(zePower);

      output->fracRemaining = 0;
      output->notFinished   = 0;
    }
  }
  else 
    if(NotDone) { /* I'm a slave */

    UINT4 n2do;
    UINT4 *alphaI, *deltaI, *psiI, *plus2crossI, *plusDotcrossI;
    REAL8 alpha, delta, psi, plus2cross, plusDotcross;

    ExchParams             *thisExch = NULL;
    ExchParams              exchDutyCycle;
    ExchParams              exchFinished;
    ExchParams              exchEvents;


    /* exchange finished message */
    exchFinished.exchObjectType  = ExchFinished;
    exchFinished.send            = 0;  /* master sends */
    exchFinished.numObjects      = 0;
    exchFinished.partnerProcNum  = 0;  /* master */

    /* exchange duty cycle information */
    exchDutyCycle.exchObjectType = ExchDataList;
    exchDutyCycle.send           = 0; /* master sends */
    exchDutyCycle.numObjects     = 1; /* i send */
    exchDutyCycle.partnerProcNum = 0; /* master */

    /* exchange event information */
    exchEvents.exchObjectType = ExchEPEvent;
    exchEvents.send           = 1; /* i send */
    exchEvents.numObjects     = 1; /* i send */
    exchEvents.partnerProcNum = 0; /* master */

    LALInitializeExchange (status->statusPtr, &thisExch, 
			   &exchDutyCycle, &initExchParams);
    CHECKSTATUSPTR (status);

    LALExchangeUINT4 (status->statusPtr, &n2do, thisExch);
    CHECKSTATUSPTR (status);

    if(n2do == -1U) {

      NotDone = 0;

      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* make sure to tell the wrapperAPI that we're done */
      output->fracRemaining = 0;
      output->notFinished   = 0;

      DETATCHSTATUSPTR (status);
      RETURN (status);

    }
    else {
      
       SkyPosition position;
       REAL4TimeSeries Doutput;
       REAL4Vector *estData;
       REAL4Vector *tfcData;
       REAL4TimeSeries tseries;
       TFPlaneParams tspec;
       Spectrogram spower;
       CList clist, list, flist;
       REAL8* powSum;
       UINT4 jCnt;

       alphaI = (UINT4 *)LALMalloc(n2do * sizeof(UINT4));
       if(!alphaI) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }
       deltaI = (UINT4 *)LALMalloc(n2do * sizeof(UINT4));
       if(!deltaI) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }
       psiI = (UINT4 *)LALMalloc(n2do * sizeof(UINT4));
       if(!psiI) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }
       plus2crossI = (UINT4 *)LALMalloc(n2do * sizeof(UINT4));
       if(!plus2crossI) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }
       plusDotcrossI = (UINT4 *)LALMalloc(n2do * sizeof(UINT4));
       if(!plusDotcrossI) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }
       powSum = (REAL8 *)LALMalloc(n2do * sizeof(REAL8));
       if(!powSum) {
	 ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
       }

       for(jCnt = 0; jCnt<n2do; jCnt++) {

	 LALExchangeUINT4 (status->statusPtr, alphaI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4 (status->statusPtr, deltaI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4 (status->statusPtr, psiI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4 (status->statusPtr, plus2crossI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4 (status->statusPtr, plusDotcrossI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

       }

       LALFinalizeExchange (status->statusPtr, &thisExch);
       CHECKSTATUSPTR (status);


       /* loop over requested sky positions */
       for(jCnt = 0; jCnt<n2do; jCnt++) {

	 powSum[jCnt] = 0.0;

         alpha = params->Alpha[alphaI[jCnt]];
         delta = params->Delta[deltaI[jCnt]];
         psi = params->Psi[psiI[jCnt]];
         plus2cross = params->plus2cross[plus2crossI[jCnt]];
         plusDotcross = params->plusDotcross[plusDotcrossI[jCnt]];


	 /***************** ESTIMATION CODE *****************************/
	 position.longitude = alpha;
	 position.latitude = delta;
	 position.system = COORDINATESYSTEM_EQUATORIAL;

	 params->IFOInfo.position = &position;
	 params->IFOInfo.polAngle = psi;

	 params->IFOInfo.plus2cross = plus2cross;
	 params->IFOInfo.plusDotcross = plusDotcross;

	 estData = NULL;
	 LALCreateVector(status->statusPtr, &estData, params->NData);
	 CHECKSTATUSPTR (status);
	 Doutput.data = estData;

	 /*
    {REAL8 tmp = cosh(acosh(3.02993)/3.0);
    printf("%g\n",tmp);}
	 */

	 LALCoherentEstimation(status->statusPtr, &Doutput, &(params->IFOInfo), &(params->IFOData));
	 CHECKSTATUSPTR (status);

	 /*
	 tfcData = NULL;
	 LALCreateVector(status->statusPtr, &tfcData, params->NData);
	 CHECKSTATUSPTR (status);

	 for(i=0; i<params->NData; i++) {
	   tfcData->data[i] = estData->data[i];
	 }
	 */

	 tfcData = estData;

	 /***************** TFCLUSTERS CODE *****************************/
if(params->SearchMethod & 1) {
	 tseries.deltaT = params->IFOData.data[0].deltaT;
	 tseries.f0 = 1.0 / params->IFOData.data[0].deltaT;
	 tseries.epoch = params->IFOData.data[0].epoch;
	 tseries.data = tfcData;

	 /* thresholds estimation */
	 THREst(status->statusPtr, &tseries, params);
	 CHECKSTATUSPTR (status);

	 /* TFCLUSTERS */
	 LALPlainSpectrogram(status->statusPtr, &tspec, &tseries, params->T);
	 CHECKSTATUSPTR (status);

	 spower.power = NULL;
	 spower.params = NULL;
	 LALComputeSpectrogram(status->statusPtr, &spower, &tspec, &tseries);
	 CHECKSTATUSPTR (status);

	 LALInitCList(status->statusPtr, &clist, &tspec); 
	 CHECKSTATUSPTR (status);

	 LALGetClusters(status->statusPtr, &clist, &spower, params->dir);
	 CHECKSTATUSPTR (status);

	 LALInitCList(status->statusPtr, &list, &tspec); 
	 CHECKSTATUSPTR (status);

	 if(clist.nclusters > 0) { /* this should be fixed in lal */
	   LALClustersPowerThreshold(status->statusPtr, &list, &clist, params->dir);
	   CHECKSTATUSPTR (status);
	 }

	 LALFreeCList(status->statusPtr, &clist);
	 CHECKSTATUSPTR (status);

	 LALInitCList(status->statusPtr, &flist, &tspec); 
	 CHECKSTATUSPTR (status);

	 CleanFList(status->statusPtr, &flist, &list, params->cminf, params->cmaxf, params->cmint, params->cmaxt);
	 CHECKSTATUSPTR (status);
      
	 LALFreeCList(status->statusPtr, &list);
	 CHECKSTATUSPTR (status);

	 powSum[jCnt] = 0.0;
	 for(i=0; i<flist.nclusters; i++) {

	   for(j=0; j<flist.sizes[i]; j++) {
	     /*
	       powSum[jCnt] += (flist.P[i][j] - params->dir->rho[flist.f[i][j]]) / params->dir->rho[flist.f[i][j]];
	     */
	     powSum[jCnt] += flist.P[i][j];
	   }
	 }

	 /*
	 printf("%g\t%g\t%g\t%g\t%g\t%g\n",alpha,delta,psi,plus2cross,plusDotcross,powSum[jCnt]);
	 */
	 
	 /*
	 powSum[jCnt] = sqrt(powSum[jCnt]);
	 */

	 LALFreeCList(status->statusPtr, &flist);
	 CHECKSTATUSPTR (status);

	 LALFreeSpecgram(status->statusPtr, &spower);
	 CHECKSTATUSPTR (status);
} else {
         if(params->SearchMethod & 4) {

           powSum[jCnt] = 0.0;
           for(i=0;i<params->NData;i++) {
	    if(fabs(tfcData->data[i]) > powSum[jCnt]) {
	  	powSum[jCnt] = fabs(tfcData->data[i]);
	    } 
           }
       }
}


        
/* clean up */

/* 
 LALDestroyVector(status->statusPtr, &tfcData);
 CHECKSTATUSPTR (status);
*/

 LALDestroyVector(status->statusPtr, &estData);
 CHECKSTATUSPTR (status);

       } /* end of loop over n2do */

       thisExch = NULL;
       LALInitializeExchange (status->statusPtr, &thisExch, 
			      &exchEvents, &initExchParams);
       CHECKSTATUSPTR (status);

       LALExchangeUINT4(status->statusPtr, &n2do, thisExch);
       CHECKSTATUSPTR (status);

       for(jCnt=0;jCnt<n2do;jCnt++) {

	 LALExchangeUINT4(status->statusPtr, alphaI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4(status->statusPtr, deltaI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4(status->statusPtr, psiI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4(status->statusPtr, plus2crossI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeUINT4(status->statusPtr, plusDotcrossI + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

	 LALExchangeREAL8 (status->statusPtr, powSum + jCnt, thisExch);
	 CHECKSTATUSPTR (status);

       }

       LALFinalizeExchange (status->statusPtr, &thisExch);
       CHECKSTATUSPTR (status);
      
       LALFree(alphaI);
       LALFree(deltaI);
       LALFree(psiI);
       LALFree(plus2crossI);
       LALFree(plusDotcrossI);
       LALFree(powSum);
      
       output->numOutput     = 0;
       output->result        = NULL;
       output->fracRemaining = 0;
       output->notFinished   = 1;
    }
    } 

  DETATCHSTATUSPTR (status);
  RETURN (status);
}


static void THREst(LALStatus *status,
		   REAL4TimeSeries *tseries, 
		   CohereSearchParams *params) {
  /* NOTE : this now modifies the input!!! */

  INITSTATUS (status, "THREst", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  /* compute thresholds */ 
  if(params->eGoal > 0.0) {

    REAL4 norm, Pb, P2b, rad, Nsigma, nPb, nP2b;
    REAL8 Pbuf;
    REAL4 *P, *dmro, *rho = NULL;
    UINT4 i, j, k, nN, nPts, nTSteps;
    RiceThresholdParams rtp;
    REAL4Vector iVec;
    RealFFTPlan *plan = NULL;
    COMPLEX8Vector *fTr = NULL;

    dmro = tseries->data->data;
    nPts = params->T * (double)params->fs;


    /*
    scaleFactor = 0.0;
    for(i=0;i<tseries->data->length;i++) {
      scaleFactor += ((REAL8)dmro[i]) * ((REAL8)dmro[i]);
    }
    scaleFactor /= (REAL8)(tseries->data->length);
    scaleFactor = sqrt(scaleFactor);

    if(scaleFactor < 1e-100) {
      for(j=0;j<nPts/2;j++) {
	params->dir->rho[j] = 1e30;
      }
      DETATCHSTATUSPTR (status);
      RETURN (status);
    }

    for(i=0;i<tseries->data->length;i++) {
      dmro[i] /= scaleFactor;
    }
    */

    Nsigma = params->Nsigma;

    rtp.bpp = params->bpp;
    rtp.eGoal = params->eGoal;
    rtp.nFreq = (UINT4)ceil(0.5*params->T*(double)params->fs);

    if(!rtp.nFreq) {
      ABORT ( status, APPLYSEARCHH_ENUMZ, APPLYSEARCHH_MSGENUMZ );
    }

    rtp.P0 = (REAL8 *)LALMalloc(rtp.nFreq * sizeof(REAL8));
    if(!rtp.P0) {ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);} 
    rtp.Q = (REAL8 *)LALMalloc(rtp.nFreq * sizeof(REAL8));
    if(!rtp.Q) {ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);} 
	
    bzero(rtp.P0, rtp.nFreq * sizeof(REAL8));
    bzero(rtp.Q, rtp.nFreq * sizeof(REAL8));

    nTSteps = (UINT4)floor((double)params->NData / (double)nPts);
      
    if(!nTSteps) {
      ABORT ( status, APPLYSEARCHH_ENUMZ, APPLYSEARCHH_MSGENUMZ );
    }
    
    iVec.length = nPts;
    
    if(!nPts) {
      ABORT ( status, APPLYSEARCHH_ENUMZ, APPLYSEARCHH_MSGENUMZ );
    }

    P = (REAL4 *)LALMalloc(sizeof(REAL4) * nTSteps * nPts/2);
    if(!P) {ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);} 


    LALCreateForwardRealFFTPlan(status->statusPtr, &plan, nPts, 0);
    CHECKSTATUSPTR (status);
      
    LALCCreateVector(status->statusPtr, &fTr, nPts/2 + 1);
    CHECKSTATUSPTR (status);

    norm = sqrt((REAL4)nPts);

    for(i=0;i<nTSteps;i++) {
      iVec.data = dmro + i * nPts;
      LALForwardRealFFT(status->statusPtr, fTr, &iVec, plan);

      for(j=1; j<nPts/2+1; j++) {
	/* note: error in RiceThreshold is absolute, not relative */
	fTr->data[j].re /= norm;
	fTr->data[j].im /= norm;

	P[(j-1)*nTSteps+i] = Pbuf = fTr->data[j].re*fTr->data[j].re + fTr->data[j].im*fTr->data[j].im;

	/* NOTE: rho will be computed from 1/T to Nyquist, i.e. not for DC */
	rtp.P0[j-1] += Pbuf;
	rtp.Q[j-1] += Pbuf * Pbuf;

      }
    }

    for(j=0; j<nPts/2; j++) {
      Pb = rtp.P0[j] / (REAL4)nTSteps;
      P2b = rtp.Q[j] / (REAL4)nTSteps;

      rad = 2.0 * Pb * Pb - P2b;

      if(rad<0.0) {
	/*	    ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL); */
	rad = 0.0;
      }

      rad = sqrt(rad);
      rtp.P0[j] = Pb - rad;
      rtp.Q[j] = rad;
    }
	
    /* rerun, computing mean & variance without last NSigma */
    for(j=0; j<nPts/2; j++) {
      nPb = nP2b = 0;
      nN = 0;

      for(k=0;k < nTSteps; k++) {
	if(P[j*nTSteps+k] < rtp.Q[j] + Nsigma * rtp.P0[j]) {
	  nPb += P[j*nTSteps+k];
	  nP2b += P[j*nTSteps+k]*P[j*nTSteps+k];
	  nN++;
	}
      }

      Pb = nPb / (REAL4)nN;
      P2b = nP2b / (REAL4)nN;

      rad = 2.0 * Pb * Pb - P2b;

      if(rad<0.0) {
	/*	    ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL); */
	rad = 0.0;
      }

      rad = sqrt(rad);
      rtp.P0[j] = Pb - rad;
      rtp.Q[j] = rad;

    }

    /* check validity of input */
    for(j=0; j<nPts/2; j++) {
      if( isnan(rtp.P0[j]) ) {
	ABORT ( status, APPLYSEARCHH_EIERR, APPLYSEARCHH_MSGEIERR);
      }
      if( rtp.P0[j] <= 0.0 ) {
	ABORT ( status, APPLYSEARCHH_EIERR, APPLYSEARCHH_MSGEIERR);
      }
      if( isnan(rtp.Q[j]) ) {
	ABORT ( status, APPLYSEARCHH_EIERR, APPLYSEARCHH_MSGEIERR);
      }
      if( rtp.Q[j] < 0.0 ) {
	ABORT ( status, APPLYSEARCHH_EIERR, APPLYSEARCHH_MSGEIERR);
      }
    }

    rho = (REAL4 *)LALMalloc(rtp.nFreq * sizeof(REAL4));
    if(!rho) {ABORT ( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);}
	
    LALTFCRiceThreshold(status->statusPtr, rho, &rtp);
    CHECKSTATUSPTR (status);

    for(j=0;j<nPts/2;j++) {
      params->dir->rho[j] = (REAL8)rho[j];
    }


    /* clean up */

    LALCDestroyVector(status->statusPtr, &fTr);
    CHECKSTATUSPTR (status);

    LALDestroyRealFFTPlan(status->statusPtr, &plan);
    CHECKSTATUSPTR (status);

    LALFree(rho);

    LALFree(rtp.P0);
    LALFree(rtp.Q);

    LALFree(P);
  } /* params->GOAL > 0 */
  else { /* no need to compute threshold: white noise */

    UINT4 j;
    UINT4 nFreq = 1 + (UINT4)ceil(0.5*params->T * (double)params->fs) - 1;
    REAL8 thr = -log(params->bpp);

    if(!nFreq || isnan(thr)) {
      ABORT ( status, APPLYSEARCHH_ENUMZ, APPLYSEARCHH_MSGENUMZ );
    }


    for(j=0;j<nFreq;j++) {
      params->dir->rho[j] = thr;
    }

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



static void CleanFList(LALStatus *status, CList *flist, CList *list, UINT4 minf, UINT4 maxf, UINT4 mint, UINT4 maxt) {
  
  UINT4 i,j;
  BOOLEAN Good = 1;

  for(i=0; i<list->nclusters; i++) {

    Good = 0;

    for(j=0; j<list->sizes[i]; j++) {

      if(list->t[i][j] >= mint &&
	 list->t[i][j] <= maxt &&
	 list->f[i][j] >= minf &&
	 list->f[i][j] <= maxf) {Good = 1; break;}

      
    }
    
    if(Good) {
      (flist->nclusters)++;

      flist->sizes = (UINT4 *)LALRealloc(flist->sizes, flist->nclusters * sizeof(UINT4));
      if(!(flist->sizes)) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      flist->sizes[flist->nclusters - 1] = list->sizes[i];

      flist->t = (UINT4 **)LALRealloc(flist->t, flist->nclusters * sizeof(UINT4 *));
      if(!(flist->t)) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }


      flist->f = (UINT4 **)LALRealloc(flist->f, flist->nclusters * sizeof(UINT4 *));
      if(!(flist->f)) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      flist->P = (REAL8 **)LALRealloc(flist->P, flist->nclusters * sizeof(REAL8 *));
      if(!(flist->P)) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }


      flist->t[flist->nclusters - 1] = (UINT4 *)LALMalloc(list->sizes[i] * sizeof(UINT4));
      if(!(flist->t[flist->nclusters - 1])) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      flist->f[flist->nclusters - 1] = (UINT4 *)LALMalloc(list->sizes[i] * sizeof(UINT4));
      if(!(flist->f[flist->nclusters - 1])) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      flist->P[flist->nclusters - 1] = (REAL8 *)LALMalloc(list->sizes[i] * sizeof(REAL8));
      if(!(flist->P[flist->nclusters - 1])) {
	ABORT ( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      memcpy(flist->t[flist->nclusters - 1], list->t[i], list->sizes[i] * sizeof(UINT4));

      memcpy(flist->f[flist->nclusters - 1], list->f[i], list->sizes[i] * sizeof(UINT4));

      memcpy(flist->P[flist->nclusters - 1], list->P[i], list->sizes[i] * sizeof(REAL8));

    }
  }

}


static void BuildOutput ( LALStatus          *stat, 
			  LALSearchOutput    *output,
			  REAL8              *zePower,
			  CohereSearchParams *params) {

  dataBase             *dbEntry, *prev;
  UINT4 i;
  SearchSummaryInfo info;
  CHAR cBuf[] ="Cohere/TFClusters";

  INITSTATUS (stat, "BuildOutput", APPLYSEARCHC);
  ATTATCHSTATUSPTR (stat);

  output->fracRemaining = 0.0;
  output->notFinished = 0;

  output->numOutput = 2;
  output->result = (outPut *) LALCalloc( output->numOutput, sizeof(outPut) );

  /* Search Summary */
  dbEntry = NULL;
 
  info.dsoName = cBuf;
  info.inputStartTime = params->IFOData.data[0].epoch;
  info.inputEndTime.gpsSeconds = params->IFOData.data[0].epoch.gpsSeconds + (UINT4)floor((REAL8)params->NData / (REAL8)params->fs);
  info.inputEndTime.gpsNanoSeconds = params->IFOData.data[0].epoch.gpsNanoSeconds + (UINT4)floor(1E9 * (REAL8)params->NData / (REAL8)params->fs - floor((REAL8)params->NData / (REAL8)params->fs));
  if(info.inputEndTime.gpsNanoSeconds >= 1000000000) {
    info.inputEndTime.gpsSeconds += (UINT4)floor((REAL8)info.inputEndTime.gpsNanoSeconds / 1E9) - 1;
    info.inputEndTime.gpsNanoSeconds -= 1000000000 * ((UINT4)floor((REAL8)info.inputEndTime.gpsNanoSeconds / 1E9) - 1);
  }

  info.outputStartTime = info.inputStartTime;
  info.outputEndTime = info.inputEndTime;
  info.numEvents = 1;
  info.numNodes = params->numSlaves;

  LALCreateSearchSummaryDataBase( stat->statusPtr, &dbEntry, &info ); 
  CHECKSTATUSPTR( stat ); 

  output->result->results = dbEntry;
  output->result->search = burst;
  output->result->significant = 1;

  /* event data */
  output->result[1].search = burst;
  output->result[1].significant = 1;

  /* zeroth entry: event id */
  dbEntry = output->result[1].results = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = NULL;

  strcpy( dbEntry->tableName, "multi_burst");
  sprintf( dbEntry->columnName, "event_id" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));
  sprintf(dbEntry->rows.chars,"multi_burstgroup:multi_burst:0");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);
      
  prev = dbEntry;
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;


  /* IFOs, 12 char total */
  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "ifos" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *) LALMalloc( 12 * dbEntry->numberRows * sizeof(CHAR) );
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = 2 * params->NIFO;
  for(i=0; i<params->NIFO; i++) {
    if(params->IFOIND[i] == LHO4K) {
      sprintf(dbEntry->rows.chars + 2*i, "H1");
    } else if(params->IFOIND[i] == LHO2K) {
      sprintf(dbEntry->rows.chars + 2*i, "H2");
    } else if(params->IFOIND[i] == LLO4K) {
      sprintf(dbEntry->rows.chars + 2*i, "L1");
    } else if(params->IFOIND[i] == VIRGO) {
      sprintf(dbEntry->rows.chars + 2*i, "Vi");
    } else if(params->IFOIND[i] == TAMA) {
      sprintf(dbEntry->rows.chars + 2*i, "T3");
    } else if(params->IFOIND[i] == GEO) {
      sprintf(dbEntry->rows.chars + 2*i, "G6");
    }
  }

  prev = dbEntry;
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  
  /* filter ID */
  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "filter_id" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *) LALMalloc( 8 * dbEntry->numberRows * sizeof(CHAR) );
  sprintf(dbEntry->rows.chars, "COH+TFC");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = 7;
  
  prev = dbEntry;
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  /* entry: start_time */
  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "start_time" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(params->cmintR8);

  prev = dbEntry;

  /* entry: start_time_ns */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "multi_burst");
  sprintf( dbEntry->columnName, "start_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(1E9*(params->cmintR8 - floor(params->cmintR8)));

  prev = dbEntry;

  /* entry: duration */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "duration" );

  dbEntry->type       = real_4;
  dbEntry->numberRows = 1;
  dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.real4) = params->cmaxtR8 - params->cmintR8;

  prev = dbEntry;

    
  /* entry: central frequency */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "central_freq" );
  dbEntry->type       = real_4;
  dbEntry->numberRows = 1;
  dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.real4) = params->ccf;

  prev = dbEntry;

    
  /* entry: bandwidth  */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "bandwidth" );

  dbEntry->type       = real_4;
  dbEntry->numberRows = 1;
  dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.real4) = params->cbw;

  prev = dbEntry;


  /* entry: size stored in amplitude */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev; 

  strcpy( dbEntry->tableName, "multi_burst" );
  sprintf( dbEntry->columnName, "amplitude" );
  dbEntry->type       = real_4;
  dbEntry->numberRows = 1;
  dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.real4) = 1.0;

  prev = dbEntry;


  /******** summ_mime ***********/
  
  /* event_id */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "summ_mime_id" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *)LALMalloc(128 * sizeof(CHAR));
  sprintf(dbEntry->rows.chars,"multi_burstgroup:multi_burst:0");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

  prev = dbEntry;

  /* comment */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "comment" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *)LALMalloc(240 * sizeof(CHAR));
  sprintf(dbEntry->rows.chars,"a:%g:%g:%u,d:%g:%g:%u,p:%g:%g:%u,2:%g:%g:%u,r:%g:%g:%u",params->AlphaMin, params->AlphaMax, params->NAlpha, params->DeltaMin, params->DeltaMax, params->NDelta, params->PsiMin, params->PsiMax, params->NPsi, params->plus2crossMin, params->plus2crossMax, params->Nplus2cross, params->plusDotcrossMin, params->plusDotcrossMax, params->NplusDotcross);
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

  prev = dbEntry;


  /* mimetype */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "mimetype" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *)LALMalloc(64 * sizeof(CHAR));
  sprintf(dbEntry->rows.chars,"COHERE");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = 6;

  prev = dbEntry;

  /* description */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "descrip" );
  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.chars = (CHAR *)LALMalloc(64 * sizeof(CHAR));
  sprintf(dbEntry->rows.chars,"output from cohere/tfclusters");
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = strlen(dbEntry->rows.chars);

  prev = dbEntry;


  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  /* entry: start_time */
  strcpy( dbEntry->tableName, "summ_mime" );
  sprintf( dbEntry->columnName, "start_time" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(params->cmintR8);

  prev = dbEntry;

  /* entry: start_time_ns */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "start_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(1E9*(params->cmintR8 - floor(params->cmintR8)));

  prev = dbEntry;  


  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  /* entry: end_time */
  strcpy( dbEntry->tableName, "summ_mime" );
  sprintf( dbEntry->columnName, "end_time" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(params->cmaxtR8);

  prev = dbEntry;

  /* entry: end_time_ns */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "end_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (INT4)floor(1E9*(params->cmaxtR8 - floor(params->cmaxtR8)));

  prev = dbEntry;  


  /* data length */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;

  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "mimedata_length");
  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  dbEntry->rowDimensions = NULL;
  *(dbEntry->rows.int4s) = (params->NAlpha + params->NDelta + params->NPsi + params->Nplus2cross + params->NplusDotcross + params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) * sizeof(REAL8);
      
  prev = dbEntry;

  
  /* data (bigEndian) */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  dbEntry->previous = prev;
      
  strcpy( dbEntry->tableName, "summ_mime");
  sprintf( dbEntry->columnName, "mimedata");
  dbEntry->type       = char_u_ptr;
  dbEntry->numberRows = 1;
  dbEntry->rows.charu = (UCHAR *) LALMalloc( (1 + params->NAlpha) * (1 + params->NDelta) * (1 + params->NPsi) * (1+params->Nplus2cross) * (1+params->NplusDotcross) * sizeof(REAL8) );
      
#ifdef linux
  endian_swap((char *)(params->Alpha),sizeof(REAL8), params->NAlpha);
  endian_swap((char *)(params->Delta),sizeof(REAL8), params->NDelta);
  endian_swap((char *)(params->Psi),sizeof(REAL8), params->NPsi);
  endian_swap((char *)(params->plus2cross),sizeof(REAL8), params->Nplus2cross);
  endian_swap((char *)(params->plusDotcross),sizeof(REAL8), params->NplusDotcross);
  endian_swap((char *)zePower, sizeof(REAL8), params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross);
#endif

  memcpy(dbEntry->rows.charu, params->Alpha, params->NAlpha * sizeof(REAL8));
  memcpy(dbEntry->rows.charu + params->NAlpha * sizeof(REAL8), params->Delta, params->NDelta * sizeof(REAL8));
  memcpy(dbEntry->rows.charu + (params->NAlpha + params->NDelta) * sizeof(REAL8), params->Psi, params->NPsi * sizeof(REAL8));
  memcpy(dbEntry->rows.charu + (params->NAlpha + params->NDelta + params->NPsi) * sizeof(REAL8), params->plus2cross, params->Nplus2cross * sizeof(REAL8));
  memcpy(dbEntry->rows.charu + (params->NAlpha + params->NDelta + params->NPsi + params->Nplus2cross) * sizeof(REAL8), params->plusDotcross, params->NplusDotcross * sizeof(REAL8));

  memcpy(dbEntry->rows.charu + (params->NAlpha + params->NDelta + params->NPsi + params->Nplus2cross + params->NplusDotcross) * sizeof(REAL8), zePower, params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross * sizeof(REAL8) );
  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  *(dbEntry->rowDimensions) = (params->NAlpha + params->NDelta + params->NPsi + params->Nplus2cross + params->NplusDotcross + params->NAlpha * params->NDelta * params->NPsi * params->Nplus2cross * params->NplusDotcross) * sizeof(REAL8);
 


  /* LAST entry */
  dbEntry->next = NULL;



  DETATCHSTATUSPTR (stat);
  RETURN (stat);
}



#ifdef linux
void endian_swap(char * pdata, int dsize, int nelements)

{

        int i,j,indx;
        char tempbyte;

        if (dsize <= 1) return;

        for (i=0; i<nelements; i++)
        {
                indx = dsize;
                for (j=0; j<dsize/2; j++)
                {
                        tempbyte = pdata[j];
                        indx = indx - 1;
                        pdata[j] = pdata[indx];
                        pdata[indx] = tempbyte;
                }

                pdata = pdata + dsize;
        }

        return;

}
#endif
