/*----------------------------------------------------------------------- 
 * 
 * File Name: BuildOutput.c
 *
 * Author: Brady, P. R., and Brown, D. A.
 * 
 * Revision: $Id: BuildOutput.c,v 1.38 2003/02/03 22:05:30 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ApplySearch.h"
#include <lal/Date.h>

NRCSID( BUILDOUTPUTC, "$Id: BuildOutput.c,v 1.38 2003/02/03 22:05:30 duncan Exp $" );

void
BuildInspiralOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    InspiralEvent      *eventList
                    )
{
  outPut               *outputArray;
  InspiralEvent        *event;
  dataBase             *prev;
  dataBase             *dbEntry;
  UINT4                 numEvents = 0;
  UINT4                 i;


  INITSTATUS (status, "BuildInspiralOutPut", BUILDOUTPUTC);

  ASSERT( output, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( eventList, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 


  /*
   *
   * print the list events as recieved by the master
   *
   */


#ifdef INSPIRAL_SO_DEBUG
  event = eventList;

  while ( event )
  {
    fprintf( stdout, "build output: event id = %d\n", 
        event->id );
    fprintf( stdout, "build output: event timeIndex = %d\n", 
        event->timeIndex );
    fprintf( stdout, "build output: time.gpsSeconds = %d\n", 
        event->time.gpsSeconds );
    fprintf( stdout, "build output: time.gpsNanoSeconds = %d\n", 
        event->time.gpsNanoSeconds );

    fprintf( stdout, "build output: tmplt.number = %d\n", 
        event->tmplt.number );
    fprintf( stdout, "build output: tmplt.mass1 = %f\n", 
        event->tmplt.mass1 );
    fprintf( stdout, "build output: tmplt.mass2 = %f\n", 
        event->tmplt.mass2 );
    fprintf( stdout, "build output: tmplt.t0 = %f\n", 
        event->tmplt.t0 );
    fprintf( stdout, "build output: tmplt.t2 = %f\n", 
        event->tmplt.t2 );

    fprintf( stdout, "build output: snrsq = %e\n", 
        event->snrsq );
    fprintf( stdout, "build output: chisq = %e\n", 
        event->chisq );
    fprintf( stdout, "build output: sigma = %e\n", 
        event->sigma );
    fprintf( stdout, "build output: effDist = %e\n", 
        event->effDist );

    fprintf( stdout, "ifo name is %c%c\n", 
        event->ifoName[0], event->ifoName[1] );
    fprintf( stdout, "channel name is %s\n", 
        event->channel );

    fflush( stdout );

    event = event->next;
  }
#endif


  /*
   *
   * count the events and create memory for the outPut array
   *
   */


  for ( event = eventList; event; event = event->next )
  {
    ++numEvents;
  }

#ifdef INSPIRAL_SO_DEBUG
  fprintf( stdout, "build output: there are %u events in the list\n", 
     numEvents  );
  fflush( stdout );
#endif

  output->result = (outPut *) 
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->optional       = NULL;
  outputArray->indexNumber    = 0;


 /* 
  *
  * alloc memory for the database linked list
  *
  */


  /* ifo name */
  dbEntry = outputArray->results = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = NULL;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "ifo" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = numEvents;

  dbEntry->rowDimensions = (UINT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rows.chars = (CHAR *) 
    LALMalloc( 2 * dbEntry->numberRows * sizeof(CHAR) );
  if ( ! dbEntry->rows.chars )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* fill the array with the ifo name */
  {
    CHAR *ifoNamePtr = dbEntry->rows.chars;

    for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
    {
      if ( ! event )
      {
        ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
      }

      if ( ! event->ifoName[0] || ! event->ifoName[1] )
      {
        ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
      }

      dbEntry->rowDimensions[i] = 2;
      strncpy( ifoNamePtr, event->ifoName, 2 * sizeof(CHAR) );
      ifoNamePtr += 2;
    }
  }

  prev = dbEntry;

  /* channel name */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "channel" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = numEvents;

  dbEntry->rowDimensions = (UINT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  {
    UINT4 channelNameLength = 
      (strlen( eventList->channel ) < MAX_DB_CHANNEL_NAME) ?
      strlen( eventList->channel ) : MAX_DB_CHANNEL_NAME;

    dbEntry->rows.chars = (CHAR *) 
      LALMalloc( channelNameLength * dbEntry->numberRows * sizeof(CHAR) );
    if ( ! dbEntry->rows.chars )
    {
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }

    /* fill the array with the channel name */
    {
      CHAR *channelNamePtr = dbEntry->rows.chars;

      for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
      {
        if ( ! event )
        {
          ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
        }

        if ( ! event->channel[0] )
        {
          ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
        }

        dbEntry->rowDimensions[i] = channelNameLength;
        strncpy( channelNamePtr, event->channel, 
            channelNameLength * sizeof(CHAR) );
        channelNamePtr += channelNameLength;
      }
    }
  }

  prev = dbEntry;

  /* search */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "search" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = numEvents;

  dbEntry->rowDimensions = (UINT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* scope the blobStr variable... */
  {
    const CHAR  blobStr[] = "template";
    size_t      blobStrLen;

    blobStrLen = strlen( blobStr );

    dbEntry->rows.chars = (CHAR *) 
      LALMalloc( dbEntry->numberRows * blobStrLen * sizeof(CHAR) );

    for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
    {
      if ( ! event )
      {
        ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
      }

      dbEntry->rowDimensions[i] = (UINT4) blobStrLen;

      strncpy( dbEntry->rows.chars + i * blobStrLen, blobStr, 
          blobStrLen * sizeof(CHAR) );
    }
  }

  prev = dbEntry;

  /* end time seconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "end_time" );
  
  dbEntry->type       = int_4s;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.int4s = (INT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.int4s[i] = event->time.gpsSeconds;
  }

  prev = dbEntry;

  /* end time nanoseconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "end_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.int4s = (INT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.int4s[i] = event->time.gpsNanoSeconds;
  }

  prev = dbEntry;

  /*  impulse_time */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "impulse_time" );
  
  dbEntry->type       = int_4s;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.int4s = (INT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.int4s[i] = event->time.gpsSeconds;
  }

  prev = dbEntry;

  /*  impulse_time_ns */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "impulse_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.int4s = (INT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.int4s[i] = event->time.gpsNanoSeconds;
  }

  prev = dbEntry;

  /* template duration */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "template_duration" );

  dbEntry->type       = real_8;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.real8 = (REAL8 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL8) );
  if ( ! dbEntry->rows.real8 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real8[i] = event->templateDuration;
  }

  prev = dbEntry;

  /* event duration */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "event_duration" );

  dbEntry->type       = real_8;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.real8 = (REAL8 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL8) );
  if ( ! dbEntry->rows.real8 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real8[i] = event->eventDuration;
  }

  prev = dbEntry;

  /* amplitude (dummy value) */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "amplitude" );

  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = 0.0;
  }

  prev = dbEntry;

  /* effective distance */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "eff_distance" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = event->effDist;
  }

  prev = dbEntry;

  /* coa_phase */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "coa_phase" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = 0.0;
  }

  prev = dbEntry;

  /* mass1 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "mass1" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = (REAL4) event->tmplt.mass1;
  }

  prev = dbEntry;

  /* mass2 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "mass2" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = (REAL4) event->tmplt.mass2;
  }

  prev = dbEntry;

  /* mchirp */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "mchirp" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    REAL4 m1, m2, eta;

    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    m1 = (REAL4) event->tmplt.mass1;
    m2 = (REAL4) event->tmplt.mass2;
    eta = (REAL4) event->tmplt.eta;
    dbEntry->rows.real4[i] = (m1 + m2) * pow( eta, 3.0/5.0 ) ;
  }

  prev = dbEntry;

  /* eta */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "eta" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = (REAL4) event->tmplt.eta;
  }

  prev = dbEntry;

  /* tau0 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "tau0" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = (REAL4) event->tmplt.t0;
  }

  prev = dbEntry;

  /* tau2 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "tau2" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = (REAL4) event->tmplt.t2;
  }

  prev = dbEntry;

  /* tau3 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "tau3" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.real4[i] = 0.0;
  }

  prev = dbEntry;

  /* tau4 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "tau4" );

  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.real4[i] = 0.0;
  }

  prev = dbEntry;

  /* tau5 */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "tau5" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.real4[i] = 0.0;
  }
  
  prev = dbEntry;

  /* ttotal */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "ttotal" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.real4[i] = 0.0;
  }

  prev = dbEntry;

  /* snr */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "snr" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = sqrt( event->snrsq );
  }

  prev = dbEntry;

  /* chisqr */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "chisq" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real4 = (REAL4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.real4[i] = event->chisq;
  }

  prev = dbEntry;

  /* chisq_dof */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "chisq_dof" );
  
  dbEntry->type       = int_4s;
  dbEntry->numberRows = numEvents;

  dbEntry->rows.int4s = (INT4 *) 
    LALMalloc( dbEntry->numberRows * sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }
    
    dbEntry->rows.int4s[i] = (INT4) event->numChisqBins;
  }

  prev = dbEntry;

  /* sigmasq */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "sngl_inspiral" );
  sprintf( dbEntry->columnName, "sigmasq" );
  
  dbEntry->type       = real_8;
  dbEntry->numberRows = numEvents;
  
  dbEntry->rows.real8 = (REAL8 *) 
    LALMalloc( dbEntry->numberRows * sizeof(REAL8) );
  if ( ! dbEntry->rows.real8 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  for ( i = 0, event = eventList; i < numEvents; ++i, event = event->next )
  {
    if ( ! event )
    {
      ABORT( status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
    }

    dbEntry->rows.real8[i] = event->sigma * event->sigma;
  }

  dbEntry->next = NULL;

  RETURN (status);
}

void
BuildREAL4TimeSeriesOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    REAL4TimeSeries    *input
    )
{
  outPut               *outputArray;
  multiDimData         *mddEntry;
  LIGOTimeGPS           stopTimeGPS;
  INT8                  stopTimeNS;


  INITSTATUS (status, "BuildREAL4TimeSeriesOutPut", BUILDOUTPUTC);
  ATTATCHSTATUSPTR( status );

  ASSERT( output, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->deltaT > 0, status, 
      APPLYSEARCHH_EDELT, APPLYSEARCHH_MSGEDELT );


  /*
   *
   * create memory for the outPut array
   *
   */


  output->result = (outPut *)
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber    = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
  if ( ! mddEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* write the name and domain of the data */
  sprintf( mddEntry->name, input->name );
  sprintf( mddEntry->units, "dimensionless" );
  mddEntry->space = timeD;
#if INSPIRAL_SO_DEBUG
  fprintf( stdout, "mddEntry->name = %s\n", mddEntry->name );
#endif

  /* data type is REAL4 */
  mddEntry->type = real_4;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dTime), 0, sizeof(gpsTimeInterval) );

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! mddEntry->dimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  mddEntry->dimensions[0]    = input->data->length;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* time series data */
  LALGPStoINT8( status->statusPtr, &stopTimeNS, &(input->epoch) );
  CHECKSTATUSPTR( status );
  stopTimeNS += (INT8) ( 1e9 * (REAL8) input->data->length * input->deltaT );
  LALINT8toGPS( status->statusPtr, &stopTimeGPS, &stopTimeNS );
  CHECKSTATUSPTR( status );
  mddEntry->range.dTime.numberSamples = input->data->length;
  mddEntry->range.dTime.startSec = input->epoch.gpsSeconds;
  mddEntry->range.dTime.startNan = input->epoch.gpsNanoSeconds;
  mddEntry->range.dTime.stopSec = stopTimeGPS.gpsSeconds;
  mddEntry->range.dTime.stopNan = stopTimeGPS.gpsNanoSeconds;
  mddEntry->range.dTime.timeStepSize = input->deltaT;

#if INSPIRAL_SO_DEBUG
  fprintf( stdout, "mddEntry->range.dTime.stopNan = %d\n",
      mddEntry->range.dTime.stopNan );
  fprintf( stdout, "mddEntry->range.dTime.stopSec = %d\n",
      mddEntry->range.dTime.stopSec );
  fprintf( stdout, "mddEntry->range.dTime.startNan = %d\n",
      mddEntry->range.dTime.startNan );
  fprintf( stdout, "mddEntry->range.dTime.startSec = %d\n",
      mddEntry->range.dTime.startSec );
  fprintf( stdout, "mddEntry->range.dTime.numberSamples = %llu\n",
      mddEntry->range.dTime.numberSamples );
  fprintf( stdout, "mddEntry->range.dTime.timeStepSize = %e\n",
      mddEntry->range.dTime.timeStepSize );
#endif

  /* allocate memory for the data */
  mddEntry->data.real4 = (REAL4 *) 
    LALCalloc( input->data->length, sizeof(REAL4) );
  if ( ! mddEntry->data.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* and copy the data there */
  memcpy( mddEntry->data.real4, input->data->data, 
      input->data->length * sizeof(REAL4) );

  /* and hope for the best */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}

void
BuildREAL4FrequencySeriesOutPut (
    LALStatus                  *status,
    LALSearchOutput            *output,
    REAL4FrequencySeries       *input
    )
{
  outPut               *outputArray;
  multiDimData         *mddEntry;


  INITSTATUS (status, "BuildREAL4FrequencySeriesOutPut", BUILDOUTPUTC);

  ASSERT( output, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->deltaF > 0, status, 
      APPLYSEARCHH_EDELT, APPLYSEARCHH_MSGEDELT );


  /*
   *
   * create memory for the outPut array
   *
   */


  output->result = (outPut *)
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber    = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
  if ( ! mddEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* write the name and domain of the data */
  sprintf( mddEntry->name, input->name );
  sprintf( mddEntry->units, "dimensionless" );
  mddEntry->space = freqD;
#if INSPIRAL_SO_DEBUG
  fprintf( stdout, "mddEntry->name = %s\n", mddEntry->name );
#endif

  /* data type is REAL4 */
  mddEntry->type = real_4;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dFreq), 0, sizeof(frequencyInterval) );

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! mddEntry->dimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  mddEntry->dimensions[0]    = input->data->length;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* freq series data start and stop time are the same */
  mddEntry->range.dFreq.numberSamples = input->data->length;
  mddEntry->range.dFreq.gpsStopTimeSec = 
    mddEntry->range.dFreq.gpsStartTimeSec = input->epoch.gpsSeconds;
  mddEntry->range.dFreq.gpsStopTimeNan = 
    mddEntry->range.dFreq.gpsStartTimeNan = input->epoch.gpsNanoSeconds;
  mddEntry->range.dFreq.freqStepSize = input->deltaF;
  mddEntry->range.dFreq.stopFreq = 
    (REAL8) (input->data->length - 1) * input->deltaF;
#if INSPIRAL_SO_DEBUG
  fprintf( stdout, "mddEntry->range.dFreq.gpsStartTimeSec = %d\n",
      mddEntry->range.dFreq.gpsStartTimeSec );
  fprintf( stdout, "mddEntry->range.dFreq.gpsStartTimeNan = %d\n",
      mddEntry->range.dFreq.gpsStartTimeNan );
#endif

  /* allocate memory for the data */
  mddEntry->data.real4 = (REAL4 *) 
    LALCalloc( input->data->length, sizeof(REAL4) );
  if ( ! mddEntry->data.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* and copy the data there */
  memcpy( mddEntry->data.real4, input->data->data, 
      input->data->length * sizeof(REAL4) );

  /* and hope for the best */
  RETURN (status);
}


void
BuildCOMPLEX8FrequencySeriesOutPut (
    LALStatus                  *status,
    LALSearchOutput            *output,
    COMPLEX8FrequencySeries    *input
    )
{
  outPut               *outputArray;
  multiDimData         *mddEntry;


  INITSTATUS (status, "BuildCOMPLEX8FrequencySeriesOutPut", BUILDOUTPUTC);

  ASSERT( output, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->data->data, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( input->deltaF > 0, status, 
      APPLYSEARCHH_EDELT, APPLYSEARCHH_MSGEDELT );


  /*
   *
   * create memory for the outPut array
   *
   */


  output->result = (outPut *)
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber    = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
  if ( ! mddEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* write the name and domain of the data */
  sprintf( mddEntry->name, input->name );
  sprintf( mddEntry->units, "dimensionless" );
  mddEntry->space = freqD;

  /* data type is REAL4 */
  mddEntry->type = complex_8;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dFreq), 0, sizeof(frequencyInterval) );

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! mddEntry->dimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  mddEntry->dimensions[0]    = input->data->length;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* freq series data start and stop time are the same */
  mddEntry->range.dFreq.numberSamples = input->data->length;
  mddEntry->range.dFreq.gpsStopTimeSec = 
    mddEntry->range.dFreq.gpsStartTimeSec = input->epoch.gpsSeconds;
  mddEntry->range.dFreq.gpsStopTimeNan = 
    mddEntry->range.dFreq.gpsStartTimeNan = input->epoch.gpsNanoSeconds;

  /* allocate memory for the data */
  mddEntry->data.complex8 = (COMPLEX8 *) 
    LALCalloc( input->data->length, sizeof(COMPLEX8) );
  if ( ! mddEntry->data.complex8 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* and copy the data there */
  memcpy( mddEntry->data.complex8, input->data->data, 
      input->data->length * sizeof(COMPLEX8) );

  /* and hope for the best */
  RETURN (status);
}


void
BuildCandleOutPut (
    LALStatus                   *status,
    LALSearchOutput             *output,
    FindChirpStandardCandle     *candle,
    SearchSummaryInfo           *info
                  )
{
  outPut               *outputArray = NULL;
  multiDimData         *mddEntry = NULL;


  INITSTATUS (status, "BuildCandleOutPut", BUILDOUTPUTC);
  ATTATCHSTATUSPTR( status );

  ASSERT( output, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( candle, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( info, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );


  /*
   *
   * create memory for the outPut array
   *
   */


  output->result = (outPut *)
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->indexNumber    = 0;
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->results        = NULL;


  /* 
   *
   * alloc memory for the optional linked list
   *
   */

  
  /* allocate memory */
  mddEntry = outputArray->optional = 
    (multiDimData *) LALCalloc( 1, sizeof(multiDimData) );
  if ( ! mddEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* write the name and domain of the data */
  sprintf( mddEntry->name, "standard_candle" );
  sprintf( mddEntry->units, "m,m,snr,sigmasq,mpc" );
  mddEntry->space = timeD;

  /* data type is REAL4 */
  mddEntry->type = real_4;
  
  /* set the interval union to zero */
  memset( &(mddEntry->range.dTime), 0, sizeof(gpsTimeInterval) );
  mddEntry->range.dTime.startSec = info->inputStartTime.gpsSeconds;
  mddEntry->range.dTime.startNan = info->inputStartTime.gpsNanoSeconds;
  mddEntry->range.dTime.stopSec  = info->inputEndTime.gpsSeconds;
  mddEntry->range.dTime.stopNan  = info->inputEndTime.gpsNanoSeconds;

  /* size of the data */
  mddEntry->numberDimensions = 1;
  mddEntry->dimensions       = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! mddEntry->dimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  mddEntry->dimensions[0] = 5;

  /* dcHistory is meaningless here */
  mddEntry->history = NULL;

  /* timestep */
  {
    REAL8       stopT;
    REAL8       startT;

    LALGPStoFloat( status->statusPtr, &startT, &(info->inputStartTime) );
    CHECKSTATUSPTR( status );
    LALGPStoFloat( status->statusPtr, &stopT, &(info->inputEndTime) );
    CHECKSTATUSPTR( status );

    mddEntry->range.dTime.timeStepSize = stopT - startT; 
  }

  /* allocate memory for the data */
  mddEntry->data.real4 = (REAL4 *) 
    LALCalloc( 5, sizeof(REAL4) );
  if ( ! mddEntry->data.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* and copy the data there */
  mddEntry->data.real4[0] = candle->tmplt.mass1;
  mddEntry->data.real4[1] = candle->tmplt.mass2;
  mddEntry->data.real4[2] = sqrt( candle->rhosq );
  mddEntry->data.real4[3] = candle->sigmasq;
  mddEntry->data.real4[4] = candle->effDistance;

  /* and hope for the best */
  DETATCHSTATUSPTR( status );
  RETURN (status);
}

void
BuildCandleDataBaseOutPut (
    LALStatus                   *status,
    LALSearchOutput             *output,
    FindChirpStandardCandle     *candle,
    SearchSummaryInfo           *info
                  )
{
  dataBase             *dbEntry = NULL;
  dataBase             *prev    = NULL;
  outPut               *outputArray = NULL;

  INITSTATUS (status, "BuildCandleDataBaseOutPut", BUILDOUTPUTC);

  ASSERT( output, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( candle, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL ); 
  ASSERT( info, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );


  /*
   *
   * create memory for the outPut array
   *
   */


  output->result = (outPut *)
    LALRealloc( output->result, ++(output->numOutput) * sizeof(outPut) );
  if ( ! output->result )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* pointer to the newly created last member of the output array */
  outputArray = output->result + output->numOutput - 1;


  /*
   *
   * fill the array of output
   *
   */


  /* this is the same for every output */
  outputArray->search         = binaryInspiral;
  outputArray->significant    = 1;
  outputArray->states         = NULL;
  outputArray->optional       = NULL;
  outputArray->indexNumber    = 0;


 /* 
  *
  * alloc memory for the database linked list
  *
  */


  /* ifo name */
  dbEntry = outputArray->results = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = NULL;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "ifo" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;

  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rows.chars = (CHAR *) LALMalloc( 2 * sizeof(CHAR) );
  if ( ! dbEntry->rows.chars )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* fill the array with the ifo name */
  dbEntry->rowDimensions[0] = 2;
  strncpy( dbEntry->rows.chars, candle->ifo, 2 * sizeof(CHAR) );

  prev = dbEntry;

  /* name */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "name" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;

  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* scope the blobStr variable... */
  {
    const CHAR  blobStr[] = "inspiral_effective_distance";
    size_t      blobStrLen;

    blobStrLen = strlen( blobStr );

    dbEntry->rows.chars = (CHAR *) LALMalloc( blobStrLen * sizeof(CHAR) );

    dbEntry->rowDimensions[0] = (UINT4) blobStrLen;

    strncpy( dbEntry->rows.chars, blobStr, blobStrLen * sizeof(CHAR) );
  }

  prev = dbEntry;

  /* program */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "program" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;

  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* scope the blobStr variable... */
  {
    const CHAR  blobStr[] = "ldasinspiral";
    size_t      blobStrLen;

    blobStrLen = strlen( blobStr );

    dbEntry->rows.chars = (CHAR *) LALMalloc( blobStrLen * sizeof(CHAR) );

    dbEntry->rowDimensions[0] = (UINT4) blobStrLen;

    strncpy( dbEntry->rows.chars, blobStr, blobStrLen * sizeof(CHAR) );
  }

  prev = dbEntry;

  /* comment */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "comment" );

  dbEntry->type       = char_s_ptr;
  dbEntry->numberRows = 1;

  dbEntry->rowDimensions = (UINT4 *) LALMalloc( sizeof(UINT4) );
  if ( ! dbEntry->rowDimensions )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  /* scope the blobStr variable... */
  {
    const CHAR  blobStr[] = "1.4_1.4_8";
    size_t      blobStrLen;

    blobStrLen = strlen( blobStr );

    dbEntry->rows.chars = (CHAR *) LALMalloc( blobStrLen * sizeof(CHAR) );

    dbEntry->rowDimensions[0] = (UINT4) blobStrLen;

    strncpy( dbEntry->rows.chars, blobStr, blobStrLen * sizeof(CHAR) );
  }

  prev = dbEntry;

  /* start time seconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "start_time" );
  
  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  dbEntry->rows.int4s[0] = info->inputStartTime.gpsSeconds;

  prev = dbEntry;

  /* start time nanoseconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "start_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;

  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  dbEntry->rows.int4s[0] = info->inputStartTime.gpsNanoSeconds;

  prev = dbEntry;

  /* end time seconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "end_time" );
  
  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;
  
  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  dbEntry->rows.int4s[0] = info->inputEndTime.gpsSeconds;

  prev = dbEntry;

  /* end time nanoseconds */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;

  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "end_time_ns" );

  dbEntry->type       = int_4s;
  dbEntry->numberRows = 1;

  dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
  if ( ! dbEntry->rows.int4s )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }

  dbEntry->rowDimensions = NULL;

  dbEntry->rows.int4s[0] = info->inputEndTime.gpsNanoSeconds;

  prev = dbEntry;

  /* value */
  dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
  if ( ! dbEntry )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  dbEntry->previous = prev;
  
  sprintf( dbEntry->tableName, "summ_value" );
  sprintf( dbEntry->columnName, "value" );
  
  dbEntry->type       = real_4;
  dbEntry->numberRows = 1;
  
  dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
  if ( ! dbEntry->rows.real4 )
  {
    ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
  }
  
  dbEntry->rowDimensions = NULL;
  
  dbEntry->rows.real4[0] = candle->effDistance;

  dbEntry->next = NULL;

  /* and hope for the best */
  RETURN (status);
}
