/*************** <lalVerbatim file="ConditionCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Condition.c,v 1.1 2003/01/02 23:20:33 jolien Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <Wave.h>
#include <BuildDB.h>


NRCSID(CONDITIONC, "$Id" );


void LALConditionData(LALStatus *status,
		      LALSearchInput *inout,
		      void *searchParams,
		      LALMPIParams *mpiParams
		      )
{
  WaveParams *params;

  INT4 one=0;
  INT4 two=1;

  INITSTATUS( status, "LALConditionData", CONDITIONC );
  ATTATCHSTATUSPTR( status );

  mpiParams=NULL;

  params=(WaveParams*)searchParams;

  /* Checking data consistency */

  if(inout->numberSequences!=2)
    {
      ABORT(status,WAVEH_ENSEQ,WAVEH_MSGENSEQ);
    }

  if(inout->sequences[one].type!=inout->sequences[two].type ||
     inout->sequences[one].type!=real_4)
    {
      ABORT(status,WAVEH_ETYPES,WAVEH_MSGETYPES);
    }
  
  if(inout->sequences[one].space!=timeD || 
     inout->sequences[two].space!=timeD)
    {
      ABORT(status,WAVEH_ESPACES,WAVEH_MSGESPACES);
    }

  if(inout->sequences[one].range.dTime.numberSamples!=
     inout->sequences[two].range.dTime.numberSamples ||
     inout->sequences[one].numberDimensions!=
     inout->sequences[two].numberDimensions || 
     inout->sequences[one].dimensions[0]!=
     inout->sequences[two].dimensions[0])
    {
      ABORT(status,WAVEH_EDIMS,WAVEH_MSGEDIMS);
    }

  /* Getting input data */

  params->start.gpsSeconds=
    inout->sequences[one].range.dTime.startSec;
  params->start.gpsNanoSeconds=
    inout->sequences[one].range.dTime.startNan;
  params->stop.gpsSeconds=
    inout->sequences[one].range.dTime.stopSec;
  params->stop.gpsNanoSeconds=
    inout->sequences[one].range.dTime.stopNan;

  params->oneIn->data->epoch.gpsSeconds=
    inout->sequences[one].range.dTime.startSec;
  params->oneIn->data->epoch.gpsNanoSeconds=
    inout->sequences[one].range.dTime.startNan;

  params->twoIn->data->epoch.gpsSeconds=
    inout->sequences[two].range.dTime.startSec;
  params->twoIn->data->epoch.gpsNanoSeconds=
    inout->sequences[two].range.dTime.startNan;

  strcpy(params->oneIn->data->name, params->nameOne);
  strcpy(params->twoIn->data->name, params->nameTwo);

  params->oneIn->data->data->data=
    inout->sequences[one].data.real4;
  params->oneIn->data->data->length=
    inout->sequences[one].range.dTime.numberSamples;
  params->oneIn->data->deltaT=
    inout->sequences[one].range.dTime.timeStepSize;

  params->twoIn->data->data->data=
    inout->sequences[two].data.real4;
  params->twoIn->data->data->length=
    inout->sequences[two].range.dTime.numberSamples;
  params->twoIn->data->deltaT=
    inout->sequences[two].range.dTime.timeStepSize;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

