/*************** <lalVerbatim file="SearchCV"> ****************
Author: Klimenko, S. and Yakushin, I.
$Id: Search.c,v 1.1 2003/01/02 23:20:34 jolien Exp $
**************** </lalVerbatim> *************************************/
#include <stdio.h>
#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <Wave.h>
#include <BuildDB.h>

NRCSID(SEARCHC, "$Id" );

void LALApplySearch(LALStatus *status,
		    LALSearchOutput *output,
		    LALSearchInput *input,
		    LALApplySearchParams *params
		    )
{

  InputPercentileWavelet oneInPT;
  InputPercentileWavelet twoInPT;

  InputPixelSwapWavelet oneInPS;
  InputPixelSwapWavelet twoInPS;

  InputPixelMixerWavelet oneInPM;
  InputPixelMixerWavelet twoInPM;

  InputCoincidenceWavelet coincidenceIn;

  InputClusterWavelet oneInCluster;
  InputClusterWavelet twoInCluster;

  WaveParams *my=(WaveParams*)params->searchParams;

  INITSTATUS( status, "LALApplySearch", SEARCHC );
  ATTATCHSTATUSPTR( status );

  /* Computations */

  switch(my->ncalls)
    {
    case 0:
      oneInPT.in=my->oneIn;
      oneInPT.nonZeroFraction=my->nonZeroFraction;
      LALPercentileWavelet(status->statusPtr, &my->oneOutPT, &oneInPT);
      CHECKSTATUSPTR(status);
      my->ncalls++;
      output->fracRemaining=0.9;
      output->notFinished=1;
      break;
    case 1:
      twoInPT.in=my->twoIn;
      twoInPT.nonZeroFraction=my->nonZeroFraction;
      LALPercentileWavelet(status->statusPtr, &my->twoOutPT, &twoInPT);
      CHECKSTATUSPTR(status);
      my->ncalls++;
      output->fracRemaining=0.8;
      output->notFinished=1;
      break;
    case 2:
      coincidenceIn.timeWindowNanoSec=my->timeWindowNanoSec;
      coincidenceIn.one=my->oneOutPT->out;
      coincidenceIn.two=my->twoOutPT->out;      
      LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOut, &coincidenceIn);
      CHECKSTATUSPTR(status);

      if(my->usePixelSwapOne && !my->usePixelSwapTwo)
	{
	  oneInPS.in=my->oneOutPT->out;
	  LALPixelSwapWavelet(status->statusPtr, &my->oneOutPS, &oneInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPS->out;
	  coincidenceIn.two=my->twoOutPT->out;
/*  	  my->twoOutPS->out=my->twoOutPT->out; */
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);
	}

      if(my->usePixelSwapTwo && !my->usePixelSwapOne)
	{
	  twoInPS.in=my->twoOutPT->out;
	  LALPixelSwapWavelet(status->statusPtr, &my->twoOutPS, &twoInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPT->out;
/*  	  my->oneOutPS->out=my->oneOutPT->out; */
	  coincidenceIn.two=my->twoOutPS->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);
	}
      if(my->usePixelSwapOne && my->usePixelSwapTwo)
	{
	  oneInPS.in=my->oneOutPT->out;
	  LALPixelSwapWavelet(status->statusPtr, &my->oneOutPS, &oneInPS);
	  CHECKSTATUSPTR(status);
	  twoInPS.in=my->twoOutPT->out;
	  LALPixelSwapWavelet(status->statusPtr, &my->twoOutPS, &twoInPS);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPS->out;
	  coincidenceIn.two=my->twoOutPS->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPS, &coincidenceIn);
	  CHECKSTATUSPTR(status);	
	}
      if(my->usePixelMixer)
	{
	  oneInPM.in=my->oneOutPT->out;
	  oneInPM.seed=my->seed;
	  LALPixelMixerWavelet(status->statusPtr, &my->oneOutPM, &oneInPM);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.one=my->oneOutPM->out;
	  twoInPM.in=my->twoOutPT->out;
	  twoInPM.seed=my->seed+1;
	  LALPixelMixerWavelet(status->statusPtr, &my->twoOutPM, &twoInPM);
	  CHECKSTATUSPTR(status);
	  coincidenceIn.two=my->twoOutPM->out;
	  LALCoincidenceWavelet(status->statusPtr, &my->coincidenceOutPM, &coincidenceIn);
	  CHECKSTATUSPTR(status);	
	}
      my->ncalls++;
      output->fracRemaining=0.7;
      output->notFinished=1;
      break;
    case 3:

      oneInCluster.w=my->coincidenceOut->one;
      oneInCluster.aura=my->aura;
      oneInCluster.minClusterSize=my->minClusterSize;
      oneInCluster.maxClusterSize=my->maxClusterSize;
      oneInCluster.original=my->oneIn;

      LALClusterWavelet(status->statusPtr, &my->oneOutCluster, &oneInCluster);
      CHECKSTATUSPTR(status);

      if(my->usePixelSwapOne || my->usePixelSwapTwo)
	{
	  oneInCluster.w=my->coincidenceOutPS->one;
	  LALClusterWavelet(status->statusPtr, &my->oneOutClusterPS, &oneInCluster);
	  CHECKSTATUSPTR(status);
	}
      if(my->usePixelMixer)
	{
	  oneInCluster.w=my->coincidenceOutPM->one;
	  LALClusterWavelet(status->statusPtr, &my->oneOutClusterPM, &oneInCluster);
	  CHECKSTATUSPTR(status);	  
	}

      my->ncalls++;
      output->fracRemaining=0.6;
      output->notFinished=1;
      break;
    case 4:
      twoInCluster.w=my->coincidenceOut->two;
      twoInCluster.aura=my->aura;
      twoInCluster.minClusterSize=my->minClusterSize;
      twoInCluster.maxClusterSize=my->maxClusterSize;
      twoInCluster.original=my->twoIn;

      LALClusterWavelet(status->statusPtr, &my->twoOutCluster, &twoInCluster);
      CHECKSTATUSPTR(status);

      if(my->usePixelSwapOne || my->usePixelSwapTwo)
	{
	  twoInCluster.w=my->coincidenceOutPS->two;
	  LALClusterWavelet(status->statusPtr, &my->twoOutClusterPS, &twoInCluster);
	  CHECKSTATUSPTR(status);
	}
      if(my->usePixelMixer)
	{
	  twoInCluster.w=my->coincidenceOutPM->two;
	  LALClusterWavelet(status->statusPtr, &my->twoOutClusterPM, &twoInCluster);
	  CHECKSTATUSPTR(status);	  
	}
      my->ncalls++;
      output->fracRemaining=0.5;      
      output->notFinished=1;
      break;
    case 5:
      my->input=input;
      /* output the results */
/*        printf("Before output\n"); fflush(stdout); */
      LALBuildOutputWave(status->statusPtr,my, output);
      CHECKSTATUSPTR(status);
/*        printf("After\n"); fflush(stdout); */
      output->notFinished=0;
      my->ncalls++;
      output->fracRemaining=0.0;
    }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
